from __future__ import annotations
import argparse

from .linear_equation_ga import run_linear_equation_ga
from .tsp_ga import run_tsp_ga
from .eight_queens_ga import run_eight_queens_ga
from .graph_coloring_ga import run_graph_coloring_ga
from .continuous_opt_ga import run_continuous_opt_ga
from .knapsack_ga import run_knapsack_ga
from .symbolic_gp import run_symbolic_gp
from .runner import fitness

